import 'package:flutter/material.dart';
import 'package:mi_tiempo/models/config.dart';
import 'package:mi_tiempo/models/prevision_meteorologica.dart';
import 'package:mi_tiempo/obtener_prevision.dart';

void main() {
  // Lanzamos MyApp
  runApp(MyApp());
}

// Nuestra aplicación es un widget sin estado, que
// contendrá una aplicación MaterialApp

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      debugShowCheckedModeBanner: false, // Eliminamos el banner de "Debug"
      theme: ThemeData(
        // Tema para la aplicación
        colorScheme: // Esquema de color
            ColorScheme.fromSwatch().copyWith(primary: Colors.blueGrey),

        fontFamily: 'Roboto', // Fuente por defecto

        textTheme: const TextTheme(
          // Propiedades específicas para los diferentes
          // estilos de texto
          headline5:
              TextStyle(fontWeight: FontWeight.w300, color: Colors.white),
          headline3:
              TextStyle(fontWeight: FontWeight.w400, color: Colors.white),
          headline2:
              TextStyle(fontWeight: FontWeight.w300, color: Colors.white),
        ),
      ),
      // Establecemos el título d ela aplicación y la página Home de ésta,
      // que será un widget de tipo MiAppTiempo.
      title: 'Mi App del tiempo',
      home: const MiAppTiempo(title: 'Mi App del Tiempo'),
    );
  }
}

class MiAppTiempo extends StatelessWidget {
  // Widget sin estado con la estructura de la aplicación (abbPar, body, etc.)

  // Constructor
  // El atributo key se usa para identificarlo en el árbol de componentes
  // El atributo title será el título de la barra superior
  const MiAppTiempo({Key? key, required this.title}) : super(key: key);
  final String title;

  @override
  Widget build(BuildContext context) {
    // Constuctor del widget
    return Scaffold(
      appBar: AppBar(
        title: Text(title),
      ),
      body: InfoTiempo(), //  Separamos el contenido principal en otro widget
    );
  }
}

// Infotiempo es un widget con estado, que utiliza _InfoTiempo
// para modelar el estado.
class InfoTiempo extends StatefulWidget {
  const InfoTiempo({Key? key}) : super(key: key);

  @override
  _InfoTiempo createState() => _InfoTiempo();
}

class _InfoTiempo extends State<InfoTiempo> {
  // El estado vendrá determinado por un objeto de tipo PrevisiónMeteorologica
  // Dado que se establecerá asíncronamente, utilizamos un future, que
  // además se inicializará más adelante. Para indicar esto utilizamos late.
  late Future<PrevisionMeteorologica> _value;

  @override
  initState() {
    // Establecemos el estado inicial
    super.initState();

    // obtenerPrevision devuelve un Future con un objeto
    // de previsión del tiempo
    _value = obtenerPrevision();
  }

// Constructor del Widget
  Widget build(BuildContext context) {
    return Container(
      // Contenedor general, con una imagen de fondo
      constraints: const BoxConstraints.expand(),
      decoration: const BoxDecoration(
          image: DecorationImage(
              image: AssetImage("assets/background.jpg"), fit: BoxFit.cover)),
      child: GestureDetector(
        // Detector de gestos
        // Detección de la velocidad en vertical
        // para detectar un swipe vertical
        // Utilizamos el atributo onPanEnd
        onPanEnd: (DragEndDetails downDetails) {
          if (downDetails.velocity.pixelsPerSecond.dy > 0) {
            setState(() {
              // Actualización de la previsión
              // a través del estado
              _value = obtenerPrevision();
            });
          }
        },
        child: Padding(
            // Añadimos un padding a los diferentes widgets
            padding: const EdgeInsets.all(24.0),
            child: Center(
                child: FutureBuilder(
              future: _value, // Resultado de la peracon asíncrona,
              // Propiedad builder del FutureBuilder: Con ella definimos
              // la forma de construir el widget cuando el Future es resuelto
              builder: (BuildContext context, AsyncSnapshot snapshot) {
                // Snapshot contiene el resultado de la resolución del Future
                if (snapshot.connectionState == ConnectionState.waiting) {
                  // Aquí el Future no se ha resuelto todavía
                  return const Center(
                    // Por tanto incluimos una animación de Espera
                    child: SizedBox(
                      height: 150.0,
                      width: 150.0,
                      child: CircularProgressIndicator(
                        color: Colors.white,
                      ),
                    ),
                  );
                } else if (snapshot.connectionState == ConnectionState.done) {
                  // Aquí ya se ha resuelto el Future
                  if (snapshot.hasError) {
                    // Comprobamos si hay error
                    return const Text('Error');
                  } else if (snapshot.hasData) {
                    // Si tiene datos, generamos un widget de tipo
                    // MyInfoWeather, proporcionándole la información de la
                    // previsión
                    return MyInfoWeather(prevision: snapshot.data);
                  } else {
                    // Si el snaphot no contiene datos, mostramos un texto
                    return const Text('Sin datos');
                  }
                } else {
                  // Mostramos cualquier otro estado
                  return Text('Estado: ${snapshot.connectionState}');
                }
              },
            ))),
      ),
    );
  }
}

// Widget para el contenido central con la previsión
class MyInfoWeather extends StatelessWidget {
  PrevisionMeteorologica prevision;
  MyInfoWeather({Key? key, required this.prevision}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    // Se tratará de una serie de items ordenados en una columna
    return Column(
      children: <Widget>[
        Image(
            // El primer elemento es la imagen con la previsión
            image: AssetImage("assets/icons/png/${prevision.imagen}"),
            height: 220.0),
        Center(
          // Texto con el municipio
          child: Text(
            prevision.municipio,
            // Con style establecemos el estilo del texto,
            // En este caso el headline3, definido en el tema de la aplicación.
            style: Theme.of(context).textTheme.headline3,
          ),
        ),
        Text(
          // Texto con la previsión del estado
          "${prevision.estado}",
          style: Theme.of(context).textTheme.headline5,
        ),
        Text(
          // Texto con la probabilidad de precipitación
          "\n Probabilidad de lluvia ${prevision.precipitacion}%",
          style: Theme.of(context).textTheme.headline5,
        ),
        Padding(
          // Bloque con las temperaturas
          padding: const EdgeInsets.all(24.0),
          child: Row(
            children: [
              Expanded(
                child: Center(
                  child: Text(
                    "${prevision.tMin}º",
                    style: Theme.of(context).textTheme.headline2,
                  ),
                ),
              ),
              Expanded(
                child: Center(
                  child: Text(
                    "${prevision.tMax}º",
                    style: Theme.of(context).textTheme.headline2,
                  ),
                ),
              ),
            ],
          ),
        ), // Añadimos el widet con la información del dia siguiente
        TomorrowWeather(
          tMax: prevision.tMaxManyana,
          tMin: prevision.tMinManyana,
          estado: prevision.imagenManyana,
          precipitacion: prevision.precipitacionManyana,
        ),
      ],
    );
  }
}

class TomorrowWeather extends StatelessWidget {
  final int tMax;
  final int tMin;
  final String estado;
  final int precipitacion;

  const TomorrowWeather(
      {Key? key,
      this.tMax = 0,
      this.tMin = 0,
      this.estado = "",
      this.precipitacion = 0})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    // Ahorase tratará de una serie de items ordenados en una fila
    return Row(
      children: <Widget>[
        Expanded(
          // La imagen dentro del Expanded se ajusta al tamaño
          child: Image(
            // El primer elemento es la imagen con la previsión
            image: AssetImage("assets/icons/png/$estado"),
          ),
        ),
        Expanded(
          child: Text(
            // Texto con la probabilidad de precipitación
            "$precipitacion%",
            style: Theme.of(context).textTheme.headline3,
          ),
        ),
        Expanded(
          child: Text(
            "$tMinº",
            style: Theme.of(context).textTheme.headline3,
          ),
        ),
        Expanded(
          child: Text(
            "$tMaxº",
            style: Theme.of(context).textTheme.headline3,
          ),
        )
      ],
    );
  }
}
